package pjvukol;

/**
 * Reprezentuje jednoduchy interval.
 * Jeho krajni hodnoty jsou take v nem, tj. jedna se o interval uzavreny.
 *
 * Pozn.: Pokud nekde potrebujete porovnat dve hodnoty double na rovnost,
 * pouzijte obycejneho operatoru =
 * (tj. nepouzivejte ten trik ze cviceni na epsilon-okoli, tady nebudeme nic pocitat,
 * takze nedojde ke ztrate presnosti).
 */
public class Interval implements Comparable<Interval> {
    /**
     * Zacatek intervalu
     */
    protected double zacatek;

    /**
     * Konec intervalu
     */
    protected double konec;

    /**
     * Konstruktor
     * @param zacatek zacatek intervalu
     * @param konec konec intervalu
     */
    public Interval(double zacatek, double konec){
        if (zacatek > konec) throw new RuntimeException("nelze");
        this.zacatek = zacatek;
        this.konec = konec;
    }
    
    /**
     * Rozhodne, zda cislo patri do intervalu.
     * Interval bereme jako uzavreny, tj. vcetne krajnich hodnot (presne, pomoci =)
     * @param cislo
     * @return true, pokud cislo do intervalu patri
     */
    public boolean obsahuje(double cislo){
        return (cislo >= zacatek && cislo <= konec);
    }

    /**
     * Prunik intervalu. Pokud se intervaly pouze dotykaji, tj. napr.
     * <1,2> a <2,3>, vysledkem je interval <2,2>
     * @param druhy druhy interval
     * @return vysledny interval nebo null
     */
    public Interval prunik(Interval druhy){
        //DOPLNTE
    }
    
    /**
     * Rozhodne, zdali se dva intervaly navzajem presahuji, tj. prunik neni nulovy.
     * Dotyk dvou intervalu v krajnich bodech je take presah.
     * @param druhy druhy interval
     * @return true, pokud se oba intervaly aspon v jednom bode prekryvaji
     */
    public boolean presah(Interval druhy){
        //DOPLNTE
    }

    /**
     * Slije dva intervaly dohromady. Tzn. ze vytvori sjednoceni, pokud se
     * intervaly prekryvaji. Pokud nelze sjednoceni udelat (intervaly jsou disjunktni),
     * vrati nul
     * @param druhy druhy interval
     * @return sjednoceni intervalu nebo null, pokud jsou intervaly disjunktni
     */
    public Interval slij(Interval druhy){
        //DOPLNTE
    }

    /**
     * Komparator na porovnani dvou intervalu.
     * Vraci kladne cislo, pokud je "this" vetsi nez druhy
     * argument, zaporny, pokud je "this" mensi, a 0 v pripade rovnosti.
     * Interval je vetsi, pokud je jeho zacatek vetsi, v pripade rovnosti je vetsi jeho konec
     * tj <0,2> je vetsi nez <0,1>
     * Rovne jsou v pripade rovnosti zacatku i konce.
     * @param o druhy interval
     * @return kladne cislo, pokud je prvni interval vetsi,
     * zaporne v pripade, ze je mensi. Nulu v pripade rovnosti.
     */
    public int compareTo(Interval o) {
        //DOPLNTE
    }

    /************ DALE ROZHODNE NEMENIT **************/

    /**
     * Spocita hash kod
     * NEMENIT!!!
     * @return hash kod
     */
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + (int) (Double.doubleToLongBits(this.zacatek) ^ (Double.doubleToLongBits(this.zacatek) >>> 32));
        hash = 59 * hash + (int) (Double.doubleToLongBits(this.konec) ^ (Double.doubleToLongBits(this.konec) >>> 32));
        return hash;
    }

    /**
     * Slouzi k urceni, zda se dva intervaly rovnaji.
     * NEMENIT!!!
     * @param druhy druhy interval
     * @return true v pripade, ze jsou objekty shodne, jinak false
     */
    @Override
    public boolean equals(Object druhy){
        if (druhy instanceof Interval) return (zacatek == ((Interval)druhy).zacatek) && (konec == ((Interval)druhy).konec);
        else return false;
    }

    /**
     * Implicitni konverze na string.
     * NEMENIT !!!
     * @return reprezentaci v podobe retezce
     */
    @Override
    public String toString(){
        return String.format("<%s,%s>", String.valueOf(zacatek), String.valueOf(konec));
    }


}
