package pjvukol;

import org.junit.Test;
import static org.junit.Assert.*;

/**
 * @author Martin Plicka
 */
public class IntervalTest {

    /**
     * Test porovnavani dvou intervalu
     * Interval je "vetsi", pokud je jeho zacatek vetsi nez zacatek druheho.
     * V pripade rovnosti se rozhoduje podle konce intervalu.
     */
    @Test
    public void testCompareTo() {
        Interval a = new Interval (0,1);
        Interval b = new Interval (0,1);
        Interval c = new Interval (1,2);
        Interval d = new Interval (0,2);
        assertEquals("maji byt stejne", a.compareTo(b), 0);
        assertEquals("maji byt stejne", b.compareTo(a), 0);
        assertTrue ("D ma byt vetsi nez A", a.compareTo(d) < 0);
        assertTrue ("D ma byt vetsi nez A", d.compareTo(a) > 0);
        assertTrue ("C ma byt vetsi nez D", d.compareTo(c) < 0);
        assertTrue ("C ma byt vetsi nez D", c.compareTo(d) > 0);
        assertTrue ("C ma byt vetsi nez A", a.compareTo(c) < 0);
        assertTrue ("C ma byt vetsi nez A", c.compareTo(a) > 0);
    }

    /**
     * Krajni hodnoty intervalu se take pocitaji.
     * Tj. interval je uzavreny.
     */
    @Test
    public void testObsahuje(){
        Interval a = new Interval (-5, 10);
        assertTrue(a.obsahuje(-5.0));
        assertFalse(a.obsahuje(-5.0000001));
        assertTrue(a.obsahuje(-4.9999999));
        assertTrue(a.obsahuje(10.0));
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(9));
        assertTrue(a.obsahuje(9.9999999999));
        assertFalse(a.obsahuje(10.000000000000001));
        assertFalse(a.obsahuje(100));
    }

    @Test
    public void testPrunik(){
        Interval a = new Interval(0,1);
        Interval b = new Interval(0,2);
        Interval c = new Interval(1,2);
        Interval d = new Interval(0.5,1.5);
        Interval e = new Interval(3,4);
        Interval f = new Interval(0,5);

        assertEquals(a.prunik(b), new Interval(0,1));
        assertEquals(b.prunik(a), new Interval(0,1));

        assertEquals(a.prunik(c), new Interval(1,1));
        assertEquals(c.prunik(a), new Interval(1,1));

        assertEquals(a.prunik(d), new Interval(0.5,1));
        assertEquals(d.prunik(a), new Interval(0.5,1));

        assertEquals(e.prunik(f), new Interval(3,4));
        assertEquals(f.prunik(e), new Interval(3,4));

        assertEquals(c.prunik(e), null);
        assertEquals(e.prunik(c), null);
    }

    @Test
    public void testPresah(){
        Interval a = new Interval(0,1);
        Interval b = new Interval(0,2);
        Interval c = new Interval(1,2);
        Interval d = new Interval(0.5,1.5);
        Interval e = new Interval(3,4);
        Interval f = new Interval(0,5);

        assertEquals(a.presah(b), true);
        assertEquals(b.presah(a), true);

        assertEquals(a.presah(c), true);
        assertEquals(c.presah(a), true);

        assertEquals(a.presah(d), true);
        assertEquals(d.presah(a), true);

        assertEquals(e.presah(f), true);
        assertEquals(f.presah(e), true);

        assertEquals(c.presah(e), false);
        assertEquals(e.presah(c), false);
    }

    @Test
    public void testSlij(){
        Interval a = new Interval(1, 2);
        Interval b = new Interval(2, 3);
        Interval c = new Interval(1.5, 2.5);
        Interval d = new Interval(2.0001, 3);

        assertEquals(a.slij(b), new Interval(1,3));
        assertEquals(b.slij(a), new Interval(1,3));

        assertEquals(a.slij(c), new Interval(1,2.5));
        assertEquals(c.slij(a), new Interval(1,2.5));

        assertEquals(b.slij(c), new Interval(1.5,3));
        assertEquals(c.slij(b), new Interval(1.5,3));

        assertEquals(a.slij(d), null);
        assertEquals(d.slij(a), null);
    }

    @Test
    public void testEquals(){
        Interval a = new Interval(0,1);
        Interval b = new Interval(0,1);
        Interval c = new Interval(0,2);
        Interval d = new Interval(0.5,1);
        assertEquals(a,b);
        assertFalse(a.equals(c));
        assertFalse(c.equals(d));
        assertFalse(a.equals(d));
        assertFalse(a.equals((Object)new Integer(3)));
        assertFalse(a.equals(null));
    }

    @Test
    public void testToString(){
        assertEquals("<0.0,1.0>",new Interval(0,1).toString());
    }

}